<!DOCTYPE html>
<!--[if lt IE 7]>      <html lang="en" ng-app="fairySlipper" class="no-js lt-ie9 lt-ie8 lt-ie7"> <![endif]-->
<!--[if IE 7]>         <html lang="en" ng-app="fairySlipper" class="no-js lt-ie9 lt-ie8"> <![endif]-->
<!--[if IE 8]>         <html lang="en" ng-app="fairySlipper" class="no-js lt-ie9"> <![endif]-->
<!--[if gt IE 8]><!--> <html lang="en" ng-app="fairySlipper" class="no-js"> <!--<![endif]-->
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <title>Fairy-Slipper</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <link rel="stylesheet" href="components/bootstrap/dist/css/bootstrap.min.css">
    <link rel="stylesheet" href="components/angular-snap/angular-snap.css">
    <link rel="stylesheet" href="components/highlightjs/styles/default.css">
    <link rel="stylesheet" href="app.css">
    <script src="components/angular/angular.js"></script>
    <script src="components/angular-route/angular-route.js"></script>
    <script src="components/angular-resource/angular-resource.js"></script>
    <script src="components/angular-animate/angular-animate.js"></script>
    <script src="components/angular-bootstrap/ui-bootstrap-tpls.js"></script>
    <script src="components/angular-snap/angular-snap.js"></script>
    <script src="components/snapjs/snap.js"></script>
    <script src="components/angular-marked/angular-marked.js"></script>
    <script src="components/marked/marked.min.js"></script>
    <script src="components/highlightjs/highlight.pack.js"></script>
    <script src="components/angular-highlightjs/angular-highlightjs.js"></script>
    <script src="components/dotjem-angular-tree/dotjem-angular-tree.js"></script>
    <script src="app.js"></script>
    <script src="browser/url.js"></script>
    <script src="browser/browser.js"></script>
    <script src="browser/index.js"></script>
  </head>
  <body ng-controller="MenuCtrl" >
    <div snap-drawer>
      <div class="dragger"
           ng-mouseenter="showSidebar()"
           ng-mouseleave="hideSidebar()"
           ng-mousedown="snapRemote.open('left');">
        <span class="glyphicon glyphicon-chevron-right"></span>
      </div>
      <div class="sidebar">
        <ul class="nav sidebar-nav">
          <li ng-repeat="service in services"
              class="">
            <span class="nav-title">{{ service.title }}</span>
            <ul class="nav sidebar-nav">
              <li ng-repeat="api in service.apis"
                  ng-class="{'selected': api.url == $routeParams.service + '/' + $routeParams.version + '/'}">
                <a href="#/{{ filterByModel }}/{{ api.url }}">
                  {{ api.version }}
                </a>

                <ul class="nav sidebar-nav">
                  <li ng-repeat="api in api.section"
                      ng-class="{'selected': api.url == $routeParams.service + '/' + $routeParams.version + '/'}">
                    <a href="#/{{ filterByModel }}/{{ api.url }}">
                      {{ api.title }}
                    </a>
                  </li>
                </ul>

              </li>
            </ul>
          </li>
        </ul>
      </div>
    </div>
    <div snap-content>
      <div class="container">
        <!--[if lt IE 7]>
        <div class="alert alert-info browsehappy">You are using an <strong>outdated</strong> browser. Please <a href="http://browsehappy.com/">upgrade your browser</a> to improve your experience.</div>
        <![endif]-->
        <div ng-view></div>
      </div>
    </div>
  </body>
</html>
