<%inherit file="layout.html" />
<%def name="title()">
    Fairy-Slipper
</%def>

<%def name="style()">
  <link rel="stylesheet" href="components/bootstrap/dist/css/bootstrap.min.css">
  <link rel="stylesheet" href="components/angular-swagger-ui/dist/css/swagger-ui.min.css">
</%def>

<%def name="javascript()">
  <script src="components/angular/angular.js"></script>
  <script src="components/angular-route/angular-route.js"></script>
  <script src="components/angular-swagger-ui/dist/scripts/swagger-ui.js"></script>
  <script src="app.js"></script>
  <script src="browser/browser.js"></script>
</%def>

  <ul class="menu">
    <li><a href="#/view1">view1</a></li>
    <li><a href="#/view2">view2</a></li>
  </ul>

  <!--[if lt IE 7]>
      <p class="browsehappy">You are using an <strong>outdated</strong> browser. Please <a href="http://browsehappy.com/">upgrade your browser</a> to improve your experience.</p>
  <![endif]-->

  <div ng-view></div>
