body, h1, h2, h3, h4, h5, h6 {
    font-family: "Open Sans",Helvetica,Arial,sans-serif;
    font-weight: 400;
}

.snap-content {
    background: #fff;
    box-shadow: 0 0 20px #bccfde;
    margin-left: 30px;
}

@media (min-width: 768px) {
  .snap-open{
      // I use 180 for my snap drawer width,
      // change this to your config
      right:180px !important;
  }
}

.sidebar-nav > li > a {
    padding: 5px 15px;

}

.sidebar-nav > li > .sidebar-nav {
    margin-left: 10px;

}

dd {
    margin-left: 10px;
}


.accordion-toggle h3 {
    margin-top: 10px;
}

a.accordion-toggle:focus, a.accordion-toggle:hover {
    text-decoration: none;
}

.operation-header h3{
    margin-top: 0;
}

.operation-header small{
    font-size: 65%;
}

.operation-method {
    display: inline-block;
    vertical-align: top;
}

.operation-title {
    display: inline-block;
}

.swagger-method {
    vertical-align: top;
    top: 0.5em;
    position: relative;
}

.content {
    margin: 10px;
}

.snap-draw {
    background: #edf2f7 none repeat scroll 0 0;
}

.sidebar {
    margin: 10px 10px 10px 30px;
}

.sidebar .nav-title{
    text-transform: uppercase;
    margin: 5px;
    display: block;
    font-weight: bold;
}

.sidebar .btn-group {
    width: 100%;
}

.sidebar .btn-group .btn{
    width: 50%;
}

.sidebar .li {
    border-bottom: 1px solid #c8d8e5;
    color: #2a4e68;
}

.sidebar a {
    color: #2a4e68;
}

.sidebar .selected > a, .sidebar .selected > a:hover, .sidebar .selected > a:focus{
    background: #496e8a;
    color: #fff;
}

.sidebar a:hover {
    background: #bccfde;
    color: #2a4e68;
}

.dragger {
    background: #496e8a;
    color: #fff;
    font-size: 24px;
    height: 100%;
    width: 30px
    top: 0;
    align-content: center;
    position: fixed !important;
}


.dragger .glyphicon {
    float: left;
    margin: 0 auto;
    top: 50%;
}

/* Disable all boarders and shadows on accordians */
.panel {
    border: 0;
    box-shadow: 0 0 0 0;
}
