<!DOCTYPE html>
<!--[if lt IE 7]>      <html lang="en" ng-app="fairySlipper" class="no-js lt-ie9 lt-ie8 lt-ie7"> <![endif]-->
<!--[if IE 7]>         <html lang="en" ng-app="fairySlipper" class="no-js lt-ie9 lt-ie8"> <![endif]-->
<!--[if IE 8]>         <html lang="en" ng-app="fairySlipper" class="no-js lt-ie9"> <![endif]-->
<!--[if gt IE 8]><!--> <html lang="en" ng-app="fairySlipper" class="no-js"> <!--<![endif]-->
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <title>${self.title()}</title>
    <meta name="description" content="">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    ${self.style()}
    ${self.javascript()}
  </head>
  <body>
    ${self.body()}
  </body>
</html>

<%def name="title()">
    Default Title
</%def>

<%def name="style()">
    
</%def>

<%def name="javascript()">
    
</%def>
